/*
 * Copyright (c) 2017 ARM Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdint.h>
#include "led_port.h"
#include "s_ns_shared.h"

#if defined (__ARMCC_VERSION) && (__ARMCC_VERSION >= 6010050)
__asm(".global __ARM_use_no_argv\n");
#endif

/**
 * \brief Run LED light on available LEDs by switching 1 LED on
 *        one after one in one direction, then backwards and
 *        start again. Actual LED switch is done by secure
 *        service request.
 *
 * \param[in] led_port_bit_length Bit length of the LED port.
 */
void led_runner(unsigned int led_port_bit_length);

/**
 * \brief Blink one LED, but actual LED switch is
 *        done by secure service request.
 */
void led_blink(void);

int main(void)
{
    const unsigned int led_port_bit_length = get_led_port_bit_length();
    if (led_port_bit_length > 1) {
        led_runner(led_port_bit_length);
    } else {
        led_blink();
    }
    return 0;
}

void led_runner(unsigned int led_port_bit_length)
{
    unsigned int result = 0;
    const unsigned int led_port_value_max = 1 << (led_port_bit_length - 1);
    const unsigned int led_port_value_min = 1;
    unsigned int led_port_value = led_port_value_min;
    unsigned int direction = 0;
    for(;;) {
        result = set_secure_led_port(led_port_value);
        /* Rotate LED as soon as secure service fulfilled the write request. */
        if (result == 0)
        {
            if (direction) {
                if (led_port_value > led_port_value_min) {
                    led_port_value = led_port_value >> 1;
                } else {
                    direction = 0;
                    led_port_value = led_port_value << 1;
                }
            } else {
                if (led_port_value < led_port_value_max) {
                    led_port_value = led_port_value << 1;
                } else {
                    direction = 1;
                    led_port_value = led_port_value >> 1;
                }
            }
        }
    }
}

void led_blink(void)
{
    unsigned int led_value = 1;
    for(;;) {
        unsigned int result = set_secure_led_port(led_value);
        if (result == 0) {
            led_value = led_value ? 0 : 1;
        }
    }
}
