/*
 * Copyright (c) 2017-2018 ARM Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <stdlib.h>

#include "cmsis.h"
#include "led_port.h"
#include "region_defs.h"
#include "s_ns_shared.h"
#include "s_secure_config.h"
#include "timeout.h"
#include "serial.h"

/* Clear LSB of the function address to indicate the function-call
   will perform the switch from secure to non-secure */
#define cmse_nsfptr_create(p)   ((__typeof__(p)) ((uint32_t) p & ~1u))

/*
 * Semihosting is a mechanism that enables code running on an ARM target
 * to communicate and use the Input/Output facilities of a host computer
 * that is running a debugger.
 * There is an issue where if you use armclang at -O0 optimisation with
 * no parameters specified in the main function, the initialisation code
 * contains a breakpoint for semihosting by default. This will stop the
 * code from running before main is reached.
 * Semihosting can be disabled by defining __ARM_use_no_argv symbol
 * (or using higher optimization level).
 */
#if defined (__ARMCC_VERSION) && (__ARMCC_VERSION >= 6010050)
__asm("  .global __ARM_use_no_argv\n");
#endif

static struct timeout_t timeout = {false, NULL};

/*
 * \brief Enables the fault handlers BusFault, UsageFault,
 *        MemManageFault and SecureFault
 */
static void enable_fault_handlers(void);

/*
 * \brief Executes non-secure code
 */
static void execute_ns_code(void);


int main(void)
{
    uint32_t delay_ms = 0;

    serial_init();
    serial_print("\nSecure Blinky is running\r\n");

    enable_fault_handlers();

    /* Configure resources between secure and non-secure worlds */
    SAU_and_IDAU_config();
    MPC_config();

    led_port_init();

    /* Initialize timeout structure */
    delay_ms = 1000; /* 1 sec */
    timeout_init(&timeout, delay_ms);

    execute_ns_code();
}

static void execute_ns_code(void)
{
    /* SCB_NS.VTOR points to the Non-secure vector table base address */
    SCB_NS->VTOR = (NS_CODE_START);

    /* Setup Main stack pointer of the non-secure code */
    uint32_t ns_msp = *(uint32_t*)(NS_CODE_START);
    __TZ_set_MSP_NS(ns_msp);

    /* The entry contains adddress of the Reset_handler (CMSIS-CORE) function */
    uintptr_t entry_ptr = *(uint32_t*)(NS_CODE_START+4);

    /* Clear LSB of the function address to indicate the function-call
       will perform the switch from secure to non-secure */
    nsfptr_t ns_entry = (nsfptr_t)cmse_nsfptr_create(entry_ptr);

    /* All changes made to memory will be effective after this point */
    __DSB();
    __ISB();

    /* Call the non-secure Reset_Handler to jump to the non-secure binary */
    ns_entry();
}

static void enable_fault_handlers(void)
{
    SCB->SHCSR |= SCB_SHCSR_USGFAULTENA_Msk
               |  SCB_SHCSR_BUSFAULTENA_Msk
               |  SCB_SHCSR_MEMFAULTENA_Msk
               |  SCB_SHCSR_SECUREFAULTENA_Msk;

    /* BFSR register setting to enable precise errors */
    SCB->CFSR |= SCB_CFSR_PRECISERR_Msk;
}

/**
 * \brief Set the requested value to the LED port as a secure service.
 *        Measure the hard-coded delay by secure timer
 *        and only set LEDs at that minimum pulse with.
 *
 * \param[in] led_port_value Value to set the LED port.
 *
 * \return 0 if LED port is set to requested value, error code otherwise
 */
unsigned int set_secure_led_port(unsigned int led_port_value)
{
    if (timeout_delay_is_elapsed(&timeout)) {
        return (set_led_port(led_port_value));
    }

    return 1;
}

