/*
 * Copyright (c) 2017-2018 ARM Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "get_port.h"
#include "led_port.h"
#include "arm_gpio_drv.h"
#include "platform_retarget_dev.h"
#include "platform_retarget_pins.h"
#include "cmsis_compiler.h"

/* GPIO[2:4] can be used on the board to drive the LEDs */
#define GPIO_LED_BIT_BASE       AHB_GPIO0_2
#define LED_PORT_LENGTH         3u

/* CLZ instruction returns 32 if no bit is set */
#define CLZ_NO_BIT_RETURN       32

static uint32_t blinking_led_mask = 0;


inline unsigned int get_led_port_bit_length(void)
{
    return LED_PORT_LENGTH;
}

void led_port_init(void)
{
    struct arm_gpio_dev_t* gpio_dev = get_gpio_dev();
    uint8_t gpio_led;

    arm_gpio_init(gpio_dev);

    /* Enable all GPIO LEDs */
    for (gpio_led=0; gpio_led<LED_PORT_LENGTH; gpio_led++) {
        (void)arm_gpio_config(gpio_dev, ARM_GPIO_ACCESS_PIN,
                             (GPIO_LED_BIT_BASE + gpio_led),
                             (ARM_GPIO_PIN_ENABLE | ARM_GPIO_OUTPUT));
        /* Set LED off by default */
        /* Setting the GPIO to 1 will turn the given LED off */
        (void)arm_gpio_write(gpio_dev, ARM_GPIO_ACCESS_PIN,
                             (GPIO_LED_BIT_BASE + gpio_led), 1);
    }
}

static void set_led(struct arm_gpio_dev_t* gpio_dev,
                    uint32_t led_mask, uint32_t value)
{
    enum arm_gpio_error_t ret;
    uint32_t led = 0;
    uint32_t gpio_led = 0;

    /* Led value is a bitmask, find the first bit which is set */
    led = __RBIT(led_mask);
    led = __CLZ(led);

    /* If CLZ returns 32 it means no bit is set */
    while (led != CLZ_NO_BIT_RETURN) {
        gpio_led = led + GPIO_LED_BIT_BASE;
        if (gpio_led >= MAX_PIN_NBR) {
            return;
        }
        ret = arm_gpio_write(gpio_dev, ARM_GPIO_ACCESS_PIN,
                             (uint8_t)gpio_led, value);
        if (ret == ARM_GPIO_ERR_NONE) {
            if (!value)  {
                /* If the LED was turned on then save the LED number */
                blinking_led_mask |= (1U << led);
            } else {
                /* LED was turned off */
                blinking_led_mask &= ~(1U << led);
            }
        }

        /* Clear the previous bit and find the next one which is set */
        led_mask &= ~(1u << led);
        led = __RBIT(led_mask);
        led = __CLZ(led);
    }
}

unsigned int set_led_port(unsigned int led_mask)
{
    struct arm_gpio_dev_t* gpio_dev = get_gpio_dev();

    /* Turn off LEDs which were previously set */
    /* Setting the GPIO to 1 will turn the given LED off */
    set_led(gpio_dev, blinking_led_mask, 1);

    /* Turn on the new LED */
    /* Setting the GPIO to 0 will turn the given LED on */
    set_led(gpio_dev, led_mask, 0);

    return 0;
}
