/*
 * Copyright (c) 2016-2017 ARM Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "apb_dualtimer_drv.h"

/* Dualtimer state definitions */
#define APB_DUALTIMER_INITIALIZED    1UL

#define APB_DUALTIMER_CTRL_EN        (0x1ul << 7)
#define APB_DUALTIMER_CTRL_MODE      (0x1ul << 6)
#define APB_DUALTIMER_CTRL_INTEN     (0x1ul << 5)
#define APB_DUALTIMER_CTRL_PRESCALE  (0x3ul << 2)
#define APB_DUALTIMER_CTRL_SIZE      (0x1ul << 1)
#define APB_DUALTIMER_CTRL_ONESHOOT  (0x1ul << 0)
#define APB_DUALTIMER_INTCLR         (0x1ul << 0)
#define APB_DUALTIMER_RAWINTSTAT     (0x1ul << 0)
#define APB_DUALTIMER_MASKINTSTAT    (0x1ul << 0)

/* DualTimer register map structure */
struct apb_dtimer_reg_map_t {
    volatile uint32_t timer1load;     /* Offset: 0x000 (R/W) Timer 1 Load */
    volatile uint32_t timer1value;    /* Offset: 0x004 (R/ ) Timer 1 Counter
                                       *                     Current Value */
    volatile uint32_t timer1ctrl;     /* Offset: 0x008 (R/W) Timer 1 Control */
    volatile uint32_t timer1intclr;   /* Offset: 0x00C ( /W) Timer 1 Interrupt
                                       *                     Clear */
    volatile uint32_t timer1ris;      /* Offset: 0x010 (R/ ) Timer 1 Raw
                                       *                     Interrupt Status */
    volatile uint32_t timer1mis;      /* Offset: 0x014 (R/ ) Timer 1 Masked
                                       *                     Interrupt Status */
    volatile uint32_t timer1bgload;   /* Offset: 0x018 (R/W) Background Load
                                       *                     Register */
    volatile uint32_t reserved0;
    volatile uint32_t timer2load;     /* Offset: 0x020 (R/W) Timer 2 Load */
    volatile uint32_t timer2value;    /* Offset: 0x024 (R/ ) Timer 2 Counter
                                       *                     Current Value */
    volatile uint32_t timer2ctrl;     /* Offset: 0x028 (R/W) Timer 2 Control */
    volatile uint32_t timer2intclr;   /* Offset: 0x02C ( /W) Timer 2 Interrupt
                                       *                     Clear */
    volatile uint32_t timer2ris;      /* Offset: 0x030 (R/ ) Timer 2 Raw
                                       *                     Interrupt Status */
    volatile uint32_t timer2mis;      /* Offset: 0x034 (R/ ) Timer 2 Masked
                                       *                     Interrupt Status */
    volatile uint32_t timer2bgload;   /* Offset: 0x038 (R/W) Background Load
                                       *                     Register */
    volatile uint32_t reserved1[945];
    volatile uint32_t itcr;           /* Offset: 0xF00 (R/W) Integration Test
                                       *                     Control Register */
    volatile uint32_t itop;           /* Offset: 0xF04 ( /W) Integration Test
                                       *                     Output Set
                                       *                     Register */
};

/*
 * \brief Translates external mode definiton to internal one.
 *
 * \param[in] more  External mode value.
 *
 * \return Returns internal mode value.
 */
static uint32_t apb_dualtimer_translate_mode(uint32_t mode)
{
    uint32_t prv_mode = 0;

    /* Check 32 bit counter */
    if (mode & APB_DUALTIMER_COUNT_32) {
        prv_mode |= APB_DUALTIMER_CTRL_SIZE;
    }

    /* Check periodic mode */
    if (mode & APB_DUALTIMER_PERIODIC) {
        prv_mode |= APB_DUALTIMER_CTRL_MODE;
    }

    /* Check one shot mode */
    if (mode & APB_DUALTIMER_ONESHOT) {
        prv_mode |= APB_DUALTIMER_CTRL_ONESHOOT;
    }

    return prv_mode;
}

/*
 * \brief Sets single timer interrupt in the dualtimer hardware.
 *
 * \param[in] dev       Pointer to the dualtimer device \ref apb_dualtimer_dev_t
 * \param[in] stimer    Single timer to be enabled \ref apb_dualtimer_t
 * \param[in] load_val  Load timer value in cycles.
 * \param[in] mode      Enable mode.
 */
static void apb_single_timer_set_interrupt(struct apb_dualtimer_dev_t* dev,
                                           enum apb_dualtimer_t stimer,
                                           uint32_t load_val, uint32_t mode)
{
    struct apb_dtimer_reg_map_t* p_dualtimer =
                                  (struct apb_dtimer_reg_map_t*) dev->cfg->base;
    uint32_t prv_mode;

    prv_mode = apb_dualtimer_translate_mode(mode);

    if(stimer == SINGLETIMER1) {
         /* Enable interrupt */
        p_dualtimer->timer1ctrl = prv_mode;

        /* Reload Value */
        dev->data->timer1_reload = load_val;
        p_dualtimer->timer1load  = load_val;

        /* Enable Counter */
        p_dualtimer->timer1ctrl |= (APB_DUALTIMER_CTRL_EN |
                                    APB_DUALTIMER_CTRL_INTEN);

    } else {
        /* Enable interrupt */
        p_dualtimer->timer2ctrl = prv_mode;

        /* Reload Value */
        dev->data->timer2_reload = load_val;
        p_dualtimer->timer2load = load_val;

        /* Enable Counter */
        p_dualtimer->timer2ctrl |= (APB_DUALTIMER_CTRL_EN |
                                    APB_DUALTIMER_CTRL_INTEN);
    }
}

void apb_dualtimer_init(struct apb_dualtimer_dev_t* dev, uint32_t load_val)
{
    uint32_t reload = APB_DUALTIMER_DEFAULT_RELOAD;

    if(load_val != 0) {
        reload = load_val;
    }

    dev->data->timer1_reload = reload;
    dev->data->timer2_reload = reload;
    dev->data->state = APB_DUALTIMER_INITIALIZED;
}

enum apb_dualtimer_error_t apb_dualtimer_enable(struct apb_dualtimer_dev_t* dev,
                                                enum apb_dualtimer_t stimer)
{
    struct apb_dtimer_reg_map_t* p_dualtimer =
                                  (struct apb_dtimer_reg_map_t*) dev->cfg->base;

    /* Dualtimer has to be already initialized */
    if(dev->data->state != APB_DUALTIMER_INITIALIZED) {
        return APB_DUALTIMER_ERR_NOT_INIT;
    }

    switch(stimer) {
    case SINGLETIMER1:
        p_dualtimer->timer1ctrl &= ~APB_DUALTIMER_CTRL_EN;
        p_dualtimer->timer1load = dev->data->timer1_reload;
        p_dualtimer->timer1ctrl = (APB_DUALTIMER_CTRL_EN |
                                   APB_DUALTIMER_CTRL_SIZE);
        break;
    case SINGLETIMER2:
        p_dualtimer->timer2ctrl &= ~APB_DUALTIMER_CTRL_EN;
        p_dualtimer->timer2load = dev->data->timer2_reload;
        p_dualtimer->timer2ctrl = (APB_DUALTIMER_CTRL_EN |
                                   APB_DUALTIMER_CTRL_SIZE);
        break;
    case BOTH_TIMERS:
        /* Disable Timer */
        p_dualtimer->timer1ctrl &= ~APB_DUALTIMER_CTRL_EN;
        p_dualtimer->timer2ctrl &= ~APB_DUALTIMER_CTRL_EN;

        /* Reload Value */
        p_dualtimer->timer1load = dev->data->timer1_reload;
        p_dualtimer->timer2load = dev->data->timer2_reload;

        /* Enable Counter */
        p_dualtimer->timer1ctrl |= APB_DUALTIMER_CTRL_EN;
        p_dualtimer->timer2ctrl |= APB_DUALTIMER_CTRL_EN;
        break;
    /* default:  The default is not defined intentionally to force the
     *           compiler to check that all the enumeration values are
     *           covered in the switch.*/
    }

    return APB_DUALTIMER_ERR_NONE;
}

void apb_dualtimer_disable(struct apb_dualtimer_dev_t* dev,
                           enum apb_dualtimer_t stimer)
{
    struct apb_dtimer_reg_map_t* p_dualtimer =
                                  (struct apb_dtimer_reg_map_t*) dev->cfg->base;

    /* Dualtimer has to be already initialized and enabled */
    if(dev->data->state == APB_DUALTIMER_INITIALIZED) {
        /* Disable Timer */
        switch (stimer) {
        case SINGLETIMER1:
            p_dualtimer->timer1ctrl &= ~APB_DUALTIMER_CTRL_EN;
            break;
        case SINGLETIMER2:
            p_dualtimer->timer2ctrl &= ~APB_DUALTIMER_CTRL_EN;
            break;
        case BOTH_TIMERS:
            p_dualtimer->timer1ctrl &= ~APB_DUALTIMER_CTRL_EN;
            p_dualtimer->timer2ctrl &= ~APB_DUALTIMER_CTRL_EN;
            break;
        /* default:  The default is not defined intentionally to force the
         *           compiler to check that all the enumeration values are
         *           covered in the switch.*/
        }
    }
}

uint32_t apb_dualtimer_is_enabled(struct apb_dualtimer_dev_t* dev,
                                  enum apb_dualtimer_t stimer)
{
    uint32_t is_enabled = 0;
    struct apb_dtimer_reg_map_t* p_dualtimer =
                                  (struct apb_dtimer_reg_map_t*) dev->cfg->base;

    /* Dualtimer has to be already initialized and enabled */
    if(dev->data->state != APB_DUALTIMER_INITIALIZED) {
        return is_enabled;
    }

    switch (stimer) {
    case SINGLETIMER1:
        is_enabled = (p_dualtimer->timer1ctrl & APB_DUALTIMER_CTRL_EN);
        break;
    case SINGLETIMER2:
        is_enabled = (p_dualtimer->timer2ctrl & APB_DUALTIMER_CTRL_EN);
        break;
    case BOTH_TIMERS:
        is_enabled = (p_dualtimer->timer1ctrl & APB_DUALTIMER_CTRL_EN);
        if(is_enabled) {
            is_enabled = (p_dualtimer->timer2ctrl &
                          APB_DUALTIMER_CTRL_EN);
        }
        break;
    /* default:  The default is not defined intentionally to force the
     *           compiler to check that all the enumeration values are
     *           covered in the switch.*/
    }

    return is_enabled;
}

uint32_t apb_dualtimer_read(struct apb_dualtimer_dev_t* dev,
                            enum apb_dualtimer_t stimer)
{
    struct apb_dtimer_reg_map_t* p_dualtimer =
                                  (struct apb_dtimer_reg_map_t*) dev->cfg->base;
    uint32_t return_value = 0;

    /* Dualtimer has to be already initialized and enabled */
    if(dev->data->state != APB_DUALTIMER_INITIALIZED) {
        return return_value;
    }

    /* Disable Timer */
    switch (stimer) {
    case SINGLETIMER1:
        if(p_dualtimer->timer1ctrl & APB_DUALTIMER_CTRL_EN) {
            return_value = (dev->data->timer1_reload -
                            p_dualtimer->timer1value);
        }
        break;
    case SINGLETIMER2:
        if(p_dualtimer->timer2ctrl & APB_DUALTIMER_CTRL_EN) {
            return_value = (dev->data->timer2_reload -
                            p_dualtimer->timer2value);
        }
        break;
    case BOTH_TIMERS:
        /* Nothing to be done in here */
        break;
    /* default:  The default is not defined intentionally to force the
     *           compiler to check that all the enumeration values are
     *           covered in the switch.*/
    }

    return return_value;
}

enum apb_dualtimer_error_t apb_dualtimer_set_interrupt(
                                                struct apb_dualtimer_dev_t* dev,
                                                enum apb_dualtimer_t stimer,
                                                uint32_t load_val,
                                                uint32_t mode)
{
    /* Dualtimer has to be already initialized */
    if(dev->data->state != APB_DUALTIMER_INITIALIZED) {
        return APB_DUALTIMER_ERR_NOT_INIT;
    }

    /* Check time us condition */
    if(load_val == 0) {
        load_val = APB_DUALTIMER_DEFAULT_RELOAD;
    }

    /* Disable Timer */
    apb_dualtimer_disable(dev, stimer);

    switch(stimer) {
    case SINGLETIMER1:
        apb_single_timer_set_interrupt(dev, stimer, load_val, mode);
        break;
    case SINGLETIMER2:
        apb_single_timer_set_interrupt(dev, stimer, load_val, mode);
        break;
    case BOTH_TIMERS:
        apb_single_timer_set_interrupt(dev, SINGLETIMER1, load_val, mode);
        apb_single_timer_set_interrupt(dev, SINGLETIMER2, load_val, mode);
        break;
    /* default:  The default is not defined intentionally to force the
     *           compiler to check that all the enumeration values are
     *           covered in the switch.*/
    }

    return APB_DUALTIMER_ERR_NONE;
}

enum apb_dualtimer_error_t apb_dualtimer_disable_interrupt(
                                                struct apb_dualtimer_dev_t* dev,
                                                enum apb_dualtimer_t stimer)
{
    struct apb_dtimer_reg_map_t* p_dualtimer =
                                  (struct apb_dtimer_reg_map_t*) dev->cfg->base;

    /* Dualtimer has to be already initialized */
    if(dev->data->state != APB_DUALTIMER_INITIALIZED) {
        return APB_DUALTIMER_ERR_NOT_INIT;
    }

    switch(stimer) {
    case SINGLETIMER1:
        p_dualtimer->timer1ctrl &= ~(APB_DUALTIMER_CTRL_EN);
        break;
    case SINGLETIMER2:
        p_dualtimer->timer2ctrl &= ~(APB_DUALTIMER_CTRL_EN);
        break;
    case BOTH_TIMERS:
        p_dualtimer->timer1ctrl &= ~(APB_DUALTIMER_CTRL_EN);
        p_dualtimer->timer2ctrl &= ~(APB_DUALTIMER_CTRL_EN);
        break;
    /* default:  The default is not defined intentionally to force the
     *           compiler to check that all the enumeration values are
     *           covered in the switch.*/
    }

    return APB_DUALTIMER_ERR_NONE;
}

enum apb_dualtimer_error_t apb_dualtimer_clear_interrupt(
                                              struct apb_dualtimer_dev_t* dev)
{
    struct apb_dtimer_reg_map_t* p_dualtimer =
                                  (struct apb_dtimer_reg_map_t*) dev->cfg->base;
    enum apb_dualtimer_t stimer;

    /* Dualtimer has to be already initialized */
    if(dev->data->state != APB_DUALTIMER_INITIALIZED) {
        return APB_DUALTIMER_ERR_NOT_INIT;
    }

    /* Get which timer has generated the IRQ */
    if(apb_dualtimer_get_irq_info(dev, &stimer) != APB_DUALTIMER_ERR_NONE) {
        return APB_DUALTIMER_ERR_NOT_INIT;
    }

    switch(stimer) {
    case SINGLETIMER1:
        p_dualtimer->timer1intclr = APB_DUALTIMER_INTCLR;
        break;
    case SINGLETIMER2:
        p_dualtimer->timer2intclr = APB_DUALTIMER_INTCLR;
        break;
    case BOTH_TIMERS:
        /* This option is not possible in here based on the
         * apb_dualtimer_get_irq_info output. So, nothing to be done
         * in here. */
         break;
    /* default:  The default is not defined intentionally to force the
     *           compiler to check that all the enumeration values are
     *           covered in the switch.*/
    }

    return APB_DUALTIMER_ERR_NONE;
}

enum apb_dualtimer_error_t apb_dualtimer_get_irq_info(
                                                struct apb_dualtimer_dev_t* dev,
                                                enum apb_dualtimer_t* stimer)
{
    struct apb_dtimer_reg_map_t* p_dualtimer =
                                  (struct apb_dtimer_reg_map_t*) dev->cfg->base;

    /* Dualtimer has to be already initialized */
    if(dev->data->state != APB_DUALTIMER_INITIALIZED) {
        return APB_DUALTIMER_ERR_NOT_INIT;
    }

    if(p_dualtimer->timer1ris) {
        *stimer = SINGLETIMER1;
    } else {
        *stimer = SINGLETIMER2;
    }

    return APB_DUALTIMER_ERR_NONE;
}

uint32_t apb_dualtimer_get_reload_value(struct apb_dualtimer_dev_t* dev,
                                        enum apb_dualtimer_t stimer)
{
    /* Dualtimer has to be already initialized */
    if(dev->data->state != APB_DUALTIMER_INITIALIZED) {
        return 0;
    }

    switch(stimer) {
    case SINGLETIMER1:
        return dev->data->timer1_reload;
    case SINGLETIMER2:
        return dev->data->timer2_reload;
    case BOTH_TIMERS:
        /* Nothing to be done here  */
        break;
    /* default:  The default is not defined intentionally to force the
     *           compiler to check that all the enumeration values are
     *           covered in the switch. */
    }

    return 0;
}
