/*
 * Copyright (c) 2016 ARM Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * \file apb_dualtimer_drv.h
 * \brief Generic driver for APB Dualtimers.
 */

#ifndef __APB_DUALTIMER_DRV_H__
#define __APB_DUALTIMER_DRV_H__

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Default reload value */
#define APB_DUALTIMER_DEFAULT_RELOAD  0xFFFFFFFF

/* APB dualtimer device configuration structure */
struct apb_dualtimer_dev_cfg_t {
    const uint32_t base;  /*!< Dualtimer base address */
};

/* APB dualtimer device data structure */
struct apb_dualtimer_dev_data_t {
    uint32_t timer1_reload;  /*!< Single timer 1 reload value */
    uint32_t timer2_reload;  /*!< Single timer 2 reload value */
    uint32_t state;          /*!< Dualtimer state */
};

/* APB timer device structure */
struct apb_dualtimer_dev_t {
    const struct apb_dualtimer_dev_cfg_t* const cfg;  /*!< Dualtimer
                                                           configuration */
    struct apb_dualtimer_dev_data_t* const data;      /*!< Dualtimer data */
};

/* Dualtimer enumeration types */
enum apb_dualtimer_error_t {
    APB_DUALTIMER_ERR_NONE = 0,    /*!< No error */
    APB_DUALTIMER_ERR_NOT_INIT,    /*!< Error dualtimer is not initialized */
    APB_DUALTIMER_ERR_NOT_ENABLE,  /*!< Error timer is not enabled */
};

enum apb_dualtimer_t {
    SINGLETIMER1 = 1,  /*!< Single timer 1 */
    SINGLETIMER2,      /*!< Single timer 2 */
    BOTH_TIMERS        /*!< Both single timers */
};

/* Enable Mode */
#define APB_DUALTIMER_COUNT_32  (1 << 0)  /*!< 32 bit Counter */
#define APB_DUALTIMER_PERIODIC  (1 << 1)  /*!< Periodic mode */
#define APB_DUALTIMER_ONESHOT   (1 << 2)  /*!< OneShot mode */

/**
 * \brief Initializes a dualtimer hardware.
 *
 * \param[in] dev       Dualtimer device structure \ref apb_dualtimer_dev_t
 * \param[in] load_val  Load timer value in cycles - 0 to reload to timer max
 *
 * \note This function doesn't check if dev is NULL.
 */
void apb_dualtimer_init(struct apb_dualtimer_dev_t* dev, uint32_t load_val);
/**
 * \brief Enables a dualtimer hardware.
 *
 * \param[in] dev     Dualtimer device structure \ref apb_dualtimer_dev_t
 * \param[in] stimer  Single timer to be enabled \ref apb_dualtimer_t
 *
 * \return Returns error code as specified in \ref apb_dualtimer_error_t
 *
 * \note This function doesn't check if dev is NULL.
 */
enum apb_dualtimer_error_t apb_dualtimer_enable(struct apb_dualtimer_dev_t* dev,
                                                enum apb_dualtimer_t stimer);
/**
 * \brief Disables a dualtimer hardware.
 *
 * \param[in] dev     Dualtimer device structure \ref apb_dualtimer_dev_t
 * \param[in] stimer  Single timer to be enabled \ref apb_dualtimer_t
 *
 * \note This function doesn't check if dev is NULL.
 */
void apb_dualtimer_disable(struct apb_dualtimer_dev_t* dev,
                           enum apb_dualtimer_t stimer);
/**
 * \brief Verifies if a single timer in the dualtimer hardware is enabled.
 *
 * \param[in] dev     Dualtimer device structure \ref apb_dualtimer_dev_t
 * \param[in] stimer  Single timer to be enabled \ref apb_dualtimer_t
 *
 * \return 1 if it is enabled, 0 otherwise.
 *
 * \note This function doesn't check if dev is NULL.
 */
uint32_t apb_dualtimer_is_enabled(struct apb_dualtimer_dev_t* dev,
                                  enum apb_dualtimer_t stimer);
/**
 * \brief Provides single timer value.
 *
 * \param[in] dev     Dualtimer device structure \ref apb_dualtimer_dev_t
 * \param[in] stimer  Single timer to be enabled \ref apb_dualtimer_t
 *
 * \return timer value in us, or 0 if the timer is disabled or BOTH_TIMERS is
 *         provided.
 *
 * \note This function doesn't check if dev is NULL.
 */
uint32_t apb_dualtimer_read(struct apb_dualtimer_dev_t* dev,
                            enum apb_dualtimer_t stimer);
/**
 * \brief Sets single timer interrupt in the dualtimer hardware.
 *
 * \param[in] dev       Dualtimer device structure \ref apb_dualtimer_dev_t
 * \param[in] stimer    Single timer to be enabled \ref apb_dualtimer_t
 * \param[in] load_val  Load timer value in cycles - 0 to reload to timer max
 * \param[in] mode      Enable mode.
 *
 * \return Returns error code as specified in \ref apb_dualtimer_error_t
 *
 * \note This function doesn't check if dev is NULL.
 */
enum apb_dualtimer_error_t apb_dualtimer_set_interrupt(
                                               struct apb_dualtimer_dev_t* dev,
                                               enum apb_dualtimer_t stimer,
                                               uint32_t load_val,
                                               uint32_t mode);
/**
 * \brief Disables single timer interrupt in the dualtimer hardware.
 *
 * \param[in] dev     Dualtimer device structure \ref apb_dualtimer_dev_t
 * \param[in] stimer  Single timer to be enabled \ref apb_dualtimer_t
 *
 * \return Returns error code as specified in \ref apb_dualtimer_error_t
 *
 * \note This function doesn't check if dev is NULL.
 */
enum apb_dualtimer_error_t apb_dualtimer_disable_interrupt(
                                               struct apb_dualtimer_dev_t* dev,
                                               enum apb_dualtimer_t stimer);
/**
 * \brief Clears timer interrupt.
 *
 * \param[in] dev  Dualtimer device structure \ref apb_dualtimer_dev_t
 *
 * \return Returns error code as specified in \ref apb_dualtimer_error_t
 *
 * \note This function doesn't check if dev is NULL.
 */
enum apb_dualtimer_error_t apb_dualtimer_clear_interrupt(
                                              struct apb_dualtimer_dev_t* dev);

/**
 * \brief Gets the single timer who caused the interrupt.
 *
 * \param[in]  dev     Dualtimer device structure \ref apb_dualtimer_dev_t
 * \param[out] stimer  The Single timer who caused the interrupt
 *                     \ref apb_dualtimer_t
 *
 * \return Returns error code as specified in \ref apb_dualtimer_error_t
 *
 * \note This function doesn't check if dev is NULL.
 */
enum apb_dualtimer_error_t apb_dualtimer_get_irq_info(
                                               struct apb_dualtimer_dev_t* dev,
                                               enum apb_dualtimer_t* stimer);
/**
 * \brief Gets the reload value of a single timer.
 *
 * \param[in] dev     Dualtimer device structure \ref apb_dualtimer_dev_t
 * \param[in] stimer  Single timer to be enabled \ref apb_dualtimer_t
 *
 * \return reload value of the selected single timer or 0 if the timer is
 *         disabled or BOTH_TIMERS is provided.
 *
 * \note This function doesn't check if dev is NULL.
 */
uint32_t apb_dualtimer_get_reload_value(struct apb_dualtimer_dev_t* dev,
                                        enum apb_dualtimer_t stimer);

#ifdef __cplusplus
}
#endif
#endif /* __APB_DUALTIMER_DRV_H__ */
